@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: nag/data.php,v 1.39.8.9 2006/08/22 15:58:06 jan Exp $
+ * $Horde: nag/data.php,v 1.39.8.16 2008/05/25 13:59:37 jan Exp $
  *
- * Copyright 2001-2006 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (GPL).  If you
+ * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ *
+ * @author Jan Schneider <jan@horde.org>
  */
 
 function _cleanup()
@@ -47,12 +49,19 @@ if (Nag::hasPermission('max_tasks') !== true &&
 }
 
 /* Field/clear name mapping. */
-$app_fields = array('name'      => _("Name"),
-                    'desc'      => _("Description"),
-                    'category'  => _("Category"),
-                    'due'       => _("Due By"),
-                    'priority'  => _("Priority"),
-                    'completed' => _("Completion Status"));
+$app_fields = array('name'           => _("Name"),
+                    'desc'           => _("Description"),
+                    'category'       => _("Category"),
+                    'assignee'       => _("Assignee"),
+                    'due'            => _("Due By"),
+                    'alarm'          => _("Alarm"),
+                    'start'          => _("Start"),
+                    'priority'       => _("Priority"),
+                    'private'        => _("Private Task"),
+                    'estimate'       => _("Estimated Time"),
+                    'completed'      => _("Completion Status"),
+                    'completed_date' => _("Completion Date"),
+                    'uid'            => _("Unique ID"));
 
 /* Date/time fields. */
 $time_fields = array('due' => 'datetime');
@@ -73,17 +82,23 @@ case 'export':
 
     /* Get the full, sorted task list. */
     $tasks = Nag::listTasks($prefs->getValue('sortby'), $prefs->getValue('sortdir'), $prefs->getValue('altsortby'));
-    if (!count($tasks)) {
+    if (is_a($tasks, 'PEAR_Error')) {
+        $notification->push($tasks);
+        $error = true;
+    } elseif (!$tasks->hasTasks()) {
         $notification->push(_("There were no tasks to export."), 'horde.message');
         $error = true;
     } else {
+        $tasks->reset();
         switch ($exportID) {
         case EXPORT_CSV:
             $data = array();
-            foreach ($tasks as $task) {
+            while ($task = $tasks->each()) {
+                $task = $task->toHash();
                 unset($task['task_id']);
                 $task['desc'] = str_replace(',', '', $task['desc']);
                 unset($task['tasklist_id']);
+                unset($task['parent']);
                 unset($task['view_link']);
                 unset($task['complete_link']);
                 unset($task['edit_link']);
@@ -95,12 +110,14 @@ case 'export':
             exit;
 
         case EXPORT_ICALENDAR:
+            require_once NAG_BASE . '/lib/version.php';
             require_once 'Horde/iCalendar.php';
-            $vtodo = &Horde_Data::singleton('vtodo');
-            $iCal = &new Horde_iCalendar();
-            foreach ($tasks as $task) {
-                $storage = &Nag_Driver::singleton($task['tasklist_id']);
-                $iCal->addComponent($storage->toiCalendar($task,$iCal));
+            $iCal = new Horde_iCalendar();
+            $iCal->setAttribute(
+                'PRODID',
+                '-//The Horde Project//Nag ' . NAG_VERSION . '//EN');
+            while ($task = $tasks->each()) {
+                $iCal->addComponent($task->toiCalendar($iCal));
             }
             $data = $iCal->exportvCalendar();
             $browser->downloadHeaders(_("tasks.ics"), 'text/calendar', false, strlen($data));
@@ -140,6 +157,7 @@ if (is_array($next_step)) {
     $storage = &Nag_Driver::singleton($_SESSION['import_data']['target']);
     $max_tasks = Nag::hasPermission('max_tasks');
     $num_tasks = Nag::countTasks();
+    $result = null;
     foreach ($next_step as $row) {
         if ($max_tasks !== true && $num_tasks >= $max_tasks) {
             $message = @htmlspecialchars(sprintf(_("You are not allowed to create more than %d tasks."), Nag::hasPermission('max_tasks')), ENT_COMPAT, NLS::getCharset());
@@ -154,7 +172,9 @@ if (is_array($next_step)) {
             if (!is_a($row, 'Horde_iCalendar_vtodo')) {
                 continue;
             }
-            $row = $storage->fromiCalendar($row);
+            $task = new Nag_Task();
+            $task->fromiCalendar($row);
+            $row = $task->toHash();
             foreach ($app_fields as $field => $null) {
                 if (!isset($row[$field])) {
                     $row[$field] = '';
@@ -162,13 +182,12 @@ if (is_array($next_step)) {
             }
         }
 
-        $result = $storage->add(!empty($row['name']) ? $row['name'] : '',
-                                !empty($row['desc']) ? $row['desc'] : '',
-                                !empty($row['due']) ? $row['due'] : '',
-                                !empty($row['priority']) ? $row['priority'] : 3,
-                                !empty($row['completed']) ? $row['completed'] : 0,
-                                !empty($row['category']) ? $row['category'] : '',
-                                !empty($row['alarm']) ? $row['alarm'] : 0);
+        $result = $storage->add($row['name'], $row['desc'], $row['start'],
+                                $row['due'], $row['priority'],
+                                $row['estimate'], $row['completed'],
+                                $row['category'], $row['alarm'], $row['uid'],
+                                isset($row['parent']) ? $row['parent'] : '',
+                                $row['private']);
         if (is_a($result, 'PEAR_Error')) {
             break;
         }
