@@ -3,18 +3,25 @@
  * Nag storage implementation for PHP's PEAR database abstraction layer.
  *
  * Required parameters:<pre>
- *   'phptype'   The database type (e.g. 'pgsql', 'mysql', etc.).
- *   'charset'   The database's internal charset.</pre>
+ *   'phptype'      The database type (e.g. 'pgsql', 'mysql', etc.).
+ *   'charset'      The database's internal charset.</pre>
  *
  * Required by some database implementations:<pre>
- *   'hostspec'  The hostname of the database server.
- *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
- *   'database'  The name of the database.
- *   'username'  The username with which to connect to the database.
- *   'password'  The password associated with 'username'.
- *   'options'   Additional options to pass to the database.
- *   'tty'       The TTY on which to connect to the database.
- *   'port'      The port on which to connect to the database.</pre>
+ *   'hostspec'     The hostname of the database server.
+ *   'protocol'     The communication protocol ('tcp', 'unix', etc.).
+ *   'database'     The name of the database.
+ *   'username'     The username with which to connect to the database.
+ *   'password'     The password associated with 'username'.
+ *   'options'      Additional options to pass to the database.
+ *   'tty'          The TTY on which to connect to the database.
+ *   'port'         The port on which to connect to the database.</pre>
+ *
+ * Optional values when using separate reading and writing servers, for example
+ * in replication settings:<pre>
+ *   'splitread'   Boolean, whether to implement the separation or not.
+ *   'read'        Array containing the parameters which are different for
+ *                 the read database connection, currently supported
+ *                 only 'hostspec' and 'port' parameters.</pre>
  *
  * Optional parameters:<pre>
  *   'table'     The name of the tasks table in 'database'.  Default is
@@ -22,9 +29,9 @@
  *
  * The table structure can be created by the scripts/sql/nag.sql script.
  *
- * $Horde: nag/lib/Driver/sql.php,v 1.60.2.19 2007/03/13 10:14:09 jan Exp $
+ * $Horde: nag/lib/Driver/sql.php,v 1.60.2.21 2008/05/25 17:02:09 jan Exp $
  *
- * See the enclosed file COPYING for license information (GPL).  If you
+ * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
@@ -41,11 +48,12 @@ class Nag_Driver_sql extends Nag_Driver {
     var $_db;
 
     /**
-     * Are we connected to the SQL server?
+     * Handle for the current database connection, used for writing. Defaults
+     * to the same handle as $_db if a separate write database is not required.
      *
-     * @var boolean
+     * @var DB
      */
-    var $_connected = false;
+    var $_write_db;
 
     /**
      * Constructs a new SQL storage object.
@@ -64,13 +72,10 @@ class Nag_Driver_sql extends Nag_Driver {
      *
      * @param string $taskId  The id of the task to retrieve.
      *
-     * @return array  The array of task attributes.
+     * @return Nag_Task  A Nag_Task object.
      */
     function get($taskId)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         /* Build the SQL query. */
         $query = sprintf('SELECT * FROM %s WHERE task_owner = ? and task_id = ?',
                          $this->_params['table']);
@@ -92,11 +97,11 @@ class Nag_Driver_sql extends Nag_Driver {
             return $row;
         }
         if ($row === null) {
-            return PEAR::raiseError(_("Not found"));
+            return PEAR::raiseError(_("Task ID not found"));
         }
 
         /* Decode and return the task. */
-        return $this->_buildTask($row);
+        return new Nag_Task($this->_buildTask($row));
     }
 
     /**
@@ -104,13 +109,10 @@ class Nag_Driver_sql extends Nag_Driver {
      *
      * @param string $uid  The UID of the task to retrieve.
      *
-     * @return array  The array of task attributes.
+     * @return Nag_Task  A Nag_Task object.
      */
     function getByUID($uid)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         /* Build the SQL query. */
         $query = sprintf('SELECT * FROM %s WHERE task_uid = ?',
                          $this->_params['table']);
@@ -131,12 +133,12 @@ class Nag_Driver_sql extends Nag_Driver {
             return $row;
         }
         if ($row === null) {
-            return PEAR::raiseError(_("Not found"));
+            return PEAR::raiseError(_("Task UID not found"));
         }
 
         /* Decode and return the task. */
         $this->_tasklist = $row['task_owner'];
-        return $this->_buildTask($row);
+        return new Nag_Task($this->_buildTask($row));
     }
 
     /**
@@ -144,49 +146,61 @@ class Nag_Driver_sql extends Nag_Driver {
      *
      * @param string $name        The name (short) of the task.
      * @param string $desc        The description (long) of the task.
+     * @param integer $start      The start date of the task.
      * @param integer $due        The due date of the task.
      * @param integer $priority   The priority of the task.
+     * @param float $estimate     The estimated time to complete the task.
      * @param integer $completed  The completion state of the task.
      * @param string $category    The category of the task.
      * @param integer $completed  The alarm associated with the task.
      * @param string $uid         A Unique Identifier for the task.
+     * @param string $parent      The parent task.
+     * @param boolean $private    Whether the task is private.
+     * @param string $owner       The owner of the event.
+     * @param string $assignee    The assignee of the event.
      *
      * @return string  The Nag ID of the new task.
      */
-    function _add($name, $desc, $due = 0, $priority = 0, $completed = 0,
-                  $category = '', $alarm = 0, $uid = null)
+    function _add($name, $desc, $start = 0, $due = 0, $priority = 0,
+                  $estimate = 0.0, $completed = 0, $category = '', $alarm = 0,
+                  $uid = null, $parent = '', $private = false, $owner = null,
+                  $assignee = null)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         $taskId = md5(uniqid(mt_rand(), true));
         if ($uid === null) {
             $uid = $this->generateUID();
         }
 
         $query = sprintf(
-            'INSERT INTO %s (task_owner, task_id, task_name, task_uid, ' .
-            'task_desc, task_due, task_priority, task_completed, ' .
-            'task_category, task_alarm) ' .
-            'VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)',
+            'INSERT INTO %s (task_owner, task_creator, task_assignee, '
+            . 'task_id, task_name, task_uid, task_desc, task_start, task_due, '
+            . 'task_priority, task_estimate, task_completed, task_category, '
+            . 'task_alarm, task_private, task_parent) '
+            . 'VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)',
             $this->_params['table']);
         $values = array($this->_tasklist,
+                        $owner,
+                        $assignee,
                         $taskId,
                         String::convertCharset($name, NLS::getCharset(), $this->_params['charset']),
                         String::convertCharset($uid, NLS::getCharset(), $this->_params['charset']),
                         String::convertCharset($desc, NLS::getCharset(), $this->_params['charset']),
+                        (int)$start,
                         (int)$due,
                         (int)$priority,
+                        number_format($estimate, 2),
                         (int)$completed,
                         String::convertCharset($category, NLS::getCharset(), $this->_params['charset']),
-                        (int)$alarm);
+                        (int)$alarm,
+                        (int)$private,
+                        String::convertCharset($parent, NLS::getCharset(), $this->_params['charset']));
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('Nag_Driver_sql::_add(): %s', $query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Attempt the insertion query. */
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
 
         /* Return an error immediately if the query failed. */
         if (is_a($result, 'PEAR_Error')) {
@@ -200,38 +214,59 @@ class Nag_Driver_sql extends Nag_Driver {
     /**
      * Modifies an existing task.
      *
-     * @param string $taskId      The task to modify.
-     * @param string $name        The name (short) of the task.
-     * @param string $desc        The description (long) of the task.
-     * @param integer $due        The due date of the task.
-     * @param integer $priority   The priority of the task.
-     * @param integer $completed  The completion state of the task.
-     * @param string $category    The category of the task.
-     * @param integer $completed  The alarm associated with the task.
+     * @param string $taskId           The task to modify.
+     * @param string $name             The name (short) of the task.
+     * @param string $desc             The description (long) of the task.
+     * @param integer $start           The start date of the task.
+     * @param integer $due             The due date of the task.
+     * @param integer $priority        The priority of the task.
+     * @param float $estimate          The estimated time to complete the task.
+     * @param integer $completed       The completion state of the task.
+     * @param string $category         The category of the task.
+     * @param integer $alarm           The alarm associated with the task.
+     * @param string $parent           The parent task.
+     * @param boolean $private         Whether the task is private.
+     * @param string $owner            The owner of the event.
+     * @param string $assignee         The assignee of the event.
+     * @param integer $completed_date  The task's completion date.
      */
-    function _modify($taskId, $name, $desc, $due = 0, $priority = 0,
-                     $completed = 0, $category = '', $alarm = 0)
+    function _modify($taskId, $name, $desc, $start = 0, $due = 0,
+                     $priority = 0, $estimate = 0.0, $completed = 0,
+                     $category = '', $alarm = 0, $parent = '',
+                     $private = false, $owner = null, $assignee = null,
+                     $completed_date = null)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         $query = sprintf('UPDATE %s SET' .
+                         ' task_creator = ?, ' .
+                         ' task_assignee = ?, ' .
                          ' task_name = ?, ' .
                          ' task_desc = ?, ' .
+                         ' task_start = ?, ' .
                          ' task_due = ?, ' .
                          ' task_priority = ?, ' .
+                         ' task_estimate = ?, ' .
                          ' task_completed = ?, ' .
+                         ' task_completed_date = ?, ' .
                          ' task_category = ?, ' .
-                         ' task_alarm = ? ' .
+                         ' task_alarm = ?, ' .
+                         ' task_parent = ?, ' .
+                         ' task_private = ? ' .
                          'WHERE task_owner = ? AND task_id = ?',
                          $this->_params['table']);
-        $values = array(String::convertCharset($name, NLS::getCharset(), $this->_params['charset']),
+        $values = array($owner,
+                        $assignee,
+                        String::convertCharset($name, NLS::getCharset(), $this->_params['charset']),
                         String::convertCharset($desc, NLS::getCharset(), $this->_params['charset']),
+                        (int)$start,
                         (int)$due,
                         (int)$priority,
+                        number_format($estimate, 2),
                         (int)$completed,
+                        (int)$completed_date,
                         String::convertCharset($category, NLS::getCharset(), $this->_params['charset']),
                         (int)$alarm,
+                        String::convertCharset($parent, NLS::getCharset(), $this->_params['charset']),
+                        (int)$private,
                         $this->_tasklist,
                         $taskId);
 
@@ -240,7 +275,7 @@ class Nag_Driver_sql extends Nag_Driver {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Attempt the update query. */
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -257,9 +292,6 @@ class Nag_Driver_sql extends Nag_Driver {
      */
     function move($taskId, $newTasklist)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         $query = sprintf('UPDATE %s SET task_owner = ? WHERE task_owner = ? AND task_id = ?',
                          $this->_params['table']);
         $values = array($newTasklist, $this->_tasklist, $taskId);
@@ -269,7 +301,7 @@ class Nag_Driver_sql extends Nag_Driver {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Attempt the move query. */
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
             return $result;
@@ -285,8 +317,6 @@ class Nag_Driver_sql extends Nag_Driver {
      */
     function _delete($taskId)
     {
-        $this->_connect();
-
         /* Get the task's details for use later. */
         $task = $this->get($taskId);
 
@@ -299,7 +329,7 @@ class Nag_Driver_sql extends Nag_Driver {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Attempt the delete query. */
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
 
         if (is_a($result, 'PEAR_Error')) {
             Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -311,11 +341,11 @@ class Nag_Driver_sql extends Nag_Driver {
 
     /**
      * Deletes all tasks from the backend.
+     *
+     * @return mixed  True on success, PEAR_Error on failure.
      */
     function deleteAll()
     {
-        $this->_connect();
-
         $query = sprintf('DELETE FROM %s WHERE task_owner = ?',
                          $this->_params['table']);
         $values = array($this->_tasklist);
@@ -325,7 +355,7 @@ class Nag_Driver_sql extends Nag_Driver {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Attempt the delete query. */
-        $result = $this->_db->query($query, $values);
+        $result = $this->_write_db->query($query, $values);
 
         return is_a($result, 'PEAR_Error') ? $result : true;
     }
@@ -334,22 +364,33 @@ class Nag_Driver_sql extends Nag_Driver {
      * Retrieves tasks from the database.
      *
      * @param integer $completed  Which tasks to retrieve (1 = all tasks,
-     *                            0 = incomplete tasks, 2 = complete tasks).
+     *                            0 = incomplete tasks, 2 = complete tasks,
+     *                            3 = future tasks, 4 = future and incomplete
+     *                            tasks).
      *
      * @return mixed  True on success, PEAR_Error on failure.
      */
     function retrieve($completed = 1)
     {
-        /* Make sure we have a valid database connection. */
-        $this->_connect();
-
         /* Build the SQL query. */
         $query = sprintf('SELECT * FROM %s WHERE task_owner = ?',
                          $this->_params['table']);
         $values = array($this->_tasklist);
-        if ($completed != 1) {
-            $query .= ' AND task_completed = ?';
-            $values[] = $completed ? 1 : 0;
+        switch ($completed) {
+        case 0:
+            $query .= ' AND task_completed = 0 AND (task_start IS NULL OR task_start = 0 OR task_start < ?)';
+            $values[] = time();
+            break;
+        case 2:
+            $query .= ' AND task_completed = 1';
+            break;
+        case 3:
+            $query .= ' AND task_completed = 0 AND task_start > ?';
+            $values[] = time();
+            break;
+        case 4:
+            $query .= ' AND task_completed = 0';
+            break;
         }
 
         /* Log the query at a DEBUG log level. */
@@ -368,20 +409,89 @@ class Nag_Driver_sql extends Nag_Driver {
             return $row;
         }
 
-        /* Store the retrieved values in a fresh $tasks list. */
-        $this->_tasks = array();
+        /* Store the retrieved values in a fresh task list. */
+        $this->tasks = new Nag_Task();
+        $dict = array();
         while ($row && !is_a($row, 'PEAR_Error')) {
-            /* Add this new task to the $tasks list. */
-            $this->_tasks[$row['task_id']] = $this->_buildTask($row);
+            $task = &new Nag_Task($this->_buildTask($row));
+
+            /* Add task directly if it is a root task, otherwise store it in
+             * the dictionary. */
+            if (empty($row['task_parent'])) {
+                $this->tasks->add($task);
+            } else {
+                $dict[$row['task_id']] = &$task;
+            }
 
             /* Advance to the new row in the result set. */
             $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
         }
         $result->free();
 
+        /* Build a tree from the subtasks. */
+        foreach (array_keys($dict) as $key) {
+            $task = &$this->tasks->get($dict[$key]->parent_id);
+            if ($task) {
+                $task->add($dict[$key]);
+            } elseif (isset($dict[$dict[$key]->parent_id])) {
+                $dict[$dict[$key]->parent_id]->add($dict[$key]);
+            } else {
+                $this->tasks->add($dict[$key]);
+            }
+        }
+
         return true;
     }
 
+    /**
+     * Retrieves sub-tasks from the database.
+     *
+     * @param string $parentId  The parent id for the sub-tasks to retrieve.
+     *
+     * @return array  List of sub-tasks.
+     */
+    function getChildren($parentId)
+    {
+        /* Build the SQL query. */
+        $query = sprintf('SELECT * FROM %s WHERE task_owner = ? AND task_parent = ?',
+                         $this->_params['table']);
+        $values = array($this->_tasklist, $parentId);
+
+        /* Log the query at a DEBUG log level. */
+        Horde::logMessage(sprintf('Nag_Driver_sql::getChildren(): %s', $query),
+                          __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+        /* Execute the query. */
+        $result = $this->_db->query($query, $values);
+
+        if (is_a($result, 'PEAR_Error')) {
+            return $result;
+        }
+
+        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
+        if (is_a($row, 'PEAR_Error')) {
+            return $row;
+        }
+
+        /* Store the retrieved values in a fresh task list. */
+        $tasks = array();
+        while ($row && !is_a($row, 'PEAR_Error')) {
+            $task = &new Nag_Task($this->_buildTask($row));
+            $children = $this->getChildren($task->id);
+            if (is_a($children, 'PEAR_Error')) {
+                return $children;
+            }
+            $task->mergeChildren($children);
+            $tasks[] = $task;
+
+            /* Advance to the new row in the result set. */
+            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
+        }
+        $result->free();
+
+        return $tasks;
+    }
+
     /**
      * Lists all alarms near $date.
      *
@@ -391,8 +501,6 @@ class Nag_Driver_sql extends Nag_Driver {
      */
     function listAlarms($date)
     {
-        $this->_connect();
-
         $q  = 'SELECT * FROM ' . $this->_params['table'];
         $q .= ' WHERE task_owner = ?';
         $q .= ' AND task_alarm > 0';
@@ -413,11 +521,13 @@ class Nag_Driver_sql extends Nag_Driver {
 
         $tasks = array();
         foreach ($qr as $row) {
-            $tasks[$row['task_id']] = $this->_buildTask($row);
+            $tasks[$row['task_id']] = new Nag_Task($this->_buildTask($row));
         }
         return $tasks;
     }
 
+    /**
+     */
     function _buildTask($row)
     {
         /* Make sure tasks always have a UID. */
@@ -432,57 +542,80 @@ class Nag_Driver_sql extends Nag_Driver {
             /* Log the query at a DEBUG log level. */
             Horde::logMessage(sprintf('Nag_Driver_sql adding missing UID: %s', $query),
                               __FILE__, __LINE__, PEAR_LOG_DEBUG);
-            $this->_db->query($query, $values);
+            $this->_write_db->query($query, $values);
         }
 
         /* Create a new task based on $row's values. */
         return array('tasklist_id' => $row['task_owner'],
                      'task_id' => $row['task_id'],
                      'uid' => String::convertCharset($row['task_uid'], $this->_params['charset']),
+                     'parent' => $row['task_parent'],
+                     'owner' => $row['task_creator'],
+                     'assignee' => $row['task_assignee'],
                      'name' => String::convertCharset($row['task_name'], $this->_params['charset']),
                      'desc' => String::convertCharset($row['task_desc'], $this->_params['charset']),
                      'category' => String::convertCharset($row['task_category'], $this->_params['charset']),
+                     'start' => $row['task_start'],
                      'due' => $row['task_due'],
                      'priority' => $row['task_priority'],
+                     'estimate' => (float)$row['task_estimate'],
                      'completed' => $row['task_completed'],
+                     'completed_date' => isset($row['task_completed_date']) ? $row['task_completed_date'] : null,
                      'alarm' => $row['task_alarm'],
-                     'flags' => 0);
+                     'private' => $row['task_private']);
     }
 
     /**
-     * Attempts to open a persistent connection to the SQL server.
+     * Attempts to open a connection to the SQL server.
      *
-     * @return boolean    True on success; exits (Horde::fatal()) on error.
+     * @return boolean  True on success, PEAR_Error on failure.
      */
-    function _connect()
+    function initialize()
     {
-        if (!$this->_connected) {
-            Horde::assertDriverConfig($this->_params, 'storage',
-                array('phptype', 'charset'));
+        Horde::assertDriverConfig($this->_params, 'storage',
+            array('phptype', 'charset'));
 
-            if (!isset($this->_params['database'])) {
-                $this->_params['database'] = '';
-            }
-            if (!isset($this->_params['username'])) {
-                $this->_params['username'] = '';
-            }
-            if (!isset($this->_params['hostspec'])) {
-                $this->_params['hostspec'] = '';
-            }
-            if (!isset($this->_params['table'])) {
-                $this->_params['table'] = 'nag_tasks';
-            }
+        if (!isset($this->_params['database'])) {
+            $this->_params['database'] = '';
+        }
+        if (!isset($this->_params['username'])) {
+            $this->_params['username'] = '';
+        }
+        if (!isset($this->_params['hostspec'])) {
+            $this->_params['hostspec'] = '';
+        }
+        if (!isset($this->_params['table'])) {
+            $this->_params['table'] = 'nag_tasks';
+        }
 
-            /* Connect to the SQL server using the supplied parameters. */
-            require_once 'DB.php';
-            $this->_db = &DB::connect($this->_params,
-                                      array('persistent' => !empty($this->_params['persistent'])));
+        /* Connect to the SQL server using the supplied parameters. */
+        require_once 'DB.php';
+        $this->_write_db = &DB::connect($this->_params,
+                                        array('persistent' => !empty($this->_params['persistent'])));
+        if (is_a($this->_write_db, 'PEAR_Error')) {
+            return $this->_write_db;
+        }
+
+        /* Set DB portability options. */
+        switch ($this->_write_db->phptype) {
+        case 'mssql':
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            break;
+        default:
+            $this->_write_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+        }
+
+        /* Check if we need to set up the read DB connection
+         * seperately. */
+        if (!empty($this->_params['splitread'])) {
+            $params = array_merge($this->_params, $this->_params['read']);
+            $this->_db = &DB::connect($params,
+                                      array('persistent' => !empty($params['persistent'])));
             if (is_a($this->_db, 'PEAR_Error')) {
-                Horde::fatal($this->_db, __FILE__, __LINE__);
+                return $this->_db;
             }
 
-            
-            // Set DB portability options.
+            /* Set DB portability options. */
             switch ($this->_db->phptype) {
             case 'mssql':
                 $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
@@ -491,8 +624,9 @@ class Nag_Driver_sql extends Nag_Driver {
                 $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
             }
 
-
-            $this->_connected = true;
+        } else {
+            /* Default to the same DB handle for the writer too. */
+            $this->_db =& $this->_write_db;
         }
 
         return true;
