@@ -28,7 +28,7 @@ define('NAG_SORT_CATEGORY', 'category');
 /**
  * Sort by owner.
  */
-define('NAG_SORT_OWNER', 'tasklist_id');
+define('NAG_SORT_OWNER', 'tasklist');
 
 /**
  * Sort in ascending order.
@@ -43,9 +43,9 @@ define('NAG_SORT_DESCEND', 1);
 /**
  * Nag Base Class.
  *
- * $Horde: nag/lib/Nag.php,v 1.124.2.12 2006/02/19 03:12:58 chuck Exp $
+ * $Horde: nag/lib/Nag.php,v 1.124.2.24 2008/05/25 17:03:12 jan Exp $
  *
- * See the enclosed file COPYING for license information (GPL).  If you
+ * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
  *
  * @author  Jon Parise <jon@horde.org>
@@ -87,23 +87,30 @@ class Nag {
 
     /**
      * Retrieves the current user's task list from storage.
+     *
      * This function will also sort the resulting list, if requested.
      *
-     * @param Nag_Driver $storage  The current storage object.
-     * @param constant $sortby     The field by which to sort
-     *                             (NAG_SORT_PRIORITY, NAG_SORT_NAME
-     *                             NAG_SORT_DUE, NAG_SORT_COMPLETION).
-     * @param constant $sortdir    The direction by which to sort
-     *                             (NAG_SORT_ASC, NAG_SORT_DESC).
-     * @param constant $altsortby  The secondary sort field.
+     * @param string $sortby      The field by which to sort
+     *                            (NAG_SORT_PRIORITY, NAG_SORT_NAME
+     *                            NAG_SORT_DUE, NAG_SORT_COMPLETION).
+     * @param integer $sortdir    The direction by which to sort
+     *                            (NAG_SORT_ASCEND, NAG_SORT_DESCEND).
+     * @param string $altsortby   The secondary sort field.
+     * @param array $tasklists    An array of tasklist to display or
+     *                            null/empty to display taskslists
+     *                            $GLOBALS['display_tasklists'].
+     * @param integer $completed  Which tasks to retrieve (1 = all tasks,
+     *                            0 = incomplete tasks, 2 = complete tasks,
+     *                            3 = future tasks, 4 = future and incomplete
+     *                            tasks).
      *
-     * @return array  A list of the requested tasks.
-     *
-     * @see Nag_Driver::listTasks()
+     * @return Nag_Task  A list of the requested tasks.
      */
     function listTasks($sortby = null,
                        $sortdir = null,
-                       $altsortby = null)
+                       $altsortby = null,
+                       $tasklists = null,
+                       $completed = null)
     {
         global $prefs, $registry;
 
@@ -117,46 +124,32 @@ class Nag {
             $altsortby = $prefs->getValue('altsortby');
         }
 
-        /* Sorting criteria for the task list. */
-        $sort_functions = array(
-            NAG_SORT_PRIORITY => 'ByPriority',
-            NAG_SORT_NAME => 'ByName',
-            NAG_SORT_CATEGORY => 'ByCategory',
-            NAG_SORT_DUE => 'ByDue',
-            NAG_SORT_COMPLETION => 'ByCompletion',
-            NAG_SORT_OWNER => 'ByOwner'
-        );
+        if (is_null($tasklists)) {
+            $tasklists = $GLOBALS['display_tasklists'];
+        }
+        if (!is_array($tasklists)) {
+            $tasklists = array($tasklists);
+        }
+        if (is_null($completed)) {
+            $completed = $prefs->getValue('show_completed');
+        }
 
-        $tasks = array();
-        foreach ($GLOBALS['display_tasklists'] as $tasklist) {
+        $tasks = new Nag_Task();
+        foreach ($tasklists as $tasklist) {
             /* Create a Nag storage instance. */
             $storage = &Nag_Driver::singleton($tasklist);
-            $storage->retrieve();
 
             /* Retrieve the tasklist from storage. */
-            $newtasks = $storage->listTasks();
-            if (is_a($newtasks, 'PEAR_Error')) {
-                return $newtasks;
+            $result = $storage->retrieve($completed);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
             }
-
-            foreach ($newtasks as $taskID => $task) {
-                $url = Util::addParameter('view.php',
-                                          array('task' => $task['task_id'],
-                                                'tasklist' => $task['tasklist_id']));
-                $newtasks[$taskID]['view_link'] = Horde::applicationUrl($url);
-
-                $url = Util::addParameter('task.php',
-                                          array('task' => $task['task_id'],
-                                                'tasklist' => $task['tasklist_id']));
-                $url = Horde::applicationUrl($url);
-                $newtasks[$taskID]['complete_link'] = Util::addParameter($url, 'actionID', 'complete_task');
-                $newtasks[$taskID]['edit_link'] = Util::addParameter($url, 'actionID', 'modify_task');
-                $newtasks[$taskID]['delete_link'] = Util::addParameter($url, 'actionID', 'delete_tasks');
-            }
-
-            $tasks = array_merge($tasks, $newtasks);
+            $tasks->mergeChildren($storage->tasks->children);
         }
 
+        /* Process all tasks. */
+        $tasks->process();
+
         /* We look for registered apis that support listAs(taskHash). */
         $apps = @unserialize($prefs->getValue('show_external'));
         if (is_array($apps)) {
@@ -169,61 +162,38 @@ class Nag {
                         if (is_a($newtasks, 'PEAR_Error')) {
                             Horde::logMessage($newtasks, __FILE__, __LINE__, PEAR_LOG_ERR);
                         } else {
-                            $tasks = array_merge($tasks, $newtasks);
+                            foreach ($newtasks as $task) {
+                                $tasks->add(new Nag_Task($task));
+                            }
                         }
                     }
                 }
             }
         }
 
-        /* Filter complete/incomplete tasks if the user doesn't want them
-         * shown. */
-        switch ($prefs->getValue('show_completed')) {
-        case 0:
-            foreach ($tasks as $taskID => $task) {
-                if (!empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-
-        case 2:
-            foreach ($tasks as $taskID => $task) {
-                if (empty($task['completed'])) {
-                    unset($tasks[$taskID]);
-                }
-            }
-            break;
-        }
-
-        /* Sort the array if we have a sort function defined for this
-         * field. */
-        if (isset($sort_functions[$sortby])) {
-            $prefix = ($sortdir == NAG_SORT_DESCEND) ? '_rsort' : '_sort';
-            uasort($tasks, array('Nag', $prefix . $sort_functions[$sortby]));
-            if (isset($sort_functions[$altsortby]) && $altsortby !== $sortby) {
-                $task_buckets = array();
-                foreach ($tasks as $id => $task) {
-                    if (!isset($task_buckets[$task[$sortby]])) {
-                        $task_buckets[$task[$sortby]] = array();
-                    }
-                    $task_buckets[$task[$sortby]][] = $task;
-                }
-                $tasks = array();
-                foreach ($task_buckets as $task_bucket) {
-                    uasort($task_bucket, array('Nag', $prefix . $sort_functions[$altsortby]));
-                    $tasks = array_merge($tasks, $task_bucket);
-                }
-            }
-        }
+        /* Sort the array. */
+        $tasks->sort($sortby, $sortdir, $altsortby);
 
         return $tasks;
     }
 
+    /**
+     * Returns a single task.
+     *
+     * @param string $tasklist  A tasklist.
+     * @param string $task      A task id.
+     *
+     * @return array  The task hash.
+     */
     function getTask($tasklist, $task)
     {
         $storage = &Nag_Driver::singleton($tasklist);
-        return $storage->get($task);
+        $task = $storage->get($task);
+        if (is_a($task, 'PEAR_Error')) {
+            return $task;
+        }
+        $task->process();
+        return $task;
     }
 
     /**
@@ -247,7 +217,7 @@ class Nag {
             $storage->retrieve();
 
             /* Retrieve the task list from storage. */
-            $count += count($storage->listTasks());
+            $count += $storage->tasks->count();
         }
 
         return $count;
@@ -256,15 +226,18 @@ class Nag {
     /**
      * Returns all the alarms active right on $date.
      *
-     * @param object $date  The start of the time range.
+     * @param integer $date  The unix epoch time to check for alarms.
      *
-     * @return array  The alarms (alarmId) active on $date.
+     * @return array  The alarms (taskId) active on $date.
      */
-    function listAlarms($date)
+    function listAlarms($date, $tasklists = null)
     {
-        $tasks = array();
+        if (is_null($tasklists)) {
+            $tasklists = $GLOBALS['display_tasklists'];
+        }
 
-        foreach ($GLOBALS['display_tasklists'] as $tasklist) {
+        $tasks = array();
+        foreach ($tasklists as $tasklist) {
             /* Create a Nag storage instance. */
             $storage = &Nag_Driver::singleton($tasklist);
 
@@ -276,7 +249,7 @@ class Nag {
 
             /* Don't show an alarm for complete tasks. */
             foreach ($newtasks as $taskID => $task) {
-                if (!empty($task['completed'])) {
+                if (!empty($task->completed)) {
                     unset($newtasks[$taskID]);
                 }
             }
@@ -298,7 +271,7 @@ class Nag {
      */
     function listTasklists($owneronly = false, $permission = PERMS_SHOW)
     {
-        $tasklists = $GLOBALS['nag_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null);
+        $tasklists = $GLOBALS['nag_shares']->listShares(Auth::getAuth(), $permission, $owneronly ? Auth::getAuth() : null, 0, 0, 'name');
         if (is_a($tasklists, 'PEAR_Error')) {
             Horde::logMessage($tasklists, __FILE__, __LINE__, PEAR_LOG_ERR);
             return array();
@@ -307,6 +280,34 @@ class Nag {
         return $tasklists;
     }
 
+    /**
+     * Filters data based on permissions.
+     *
+     * @param array $in            The data we want filtered.
+     * @param string $filter       What type of data we are filtering.
+     * @param integer $permission  The PERMS_* constant we will filter on.
+     *
+     * @return array  The filtered data.
+     */
+    function permissionsFilter($in, $permission = PERMS_READ)
+    {
+        // FIXME: Must find a way to check individual tasklists for
+        // permission.  Can't specify attributes as it does not check for the
+        // 'key' attribute, only 'name' and 'value'.
+        return $in;
+
+        // Broken code:
+        $out = array();
+
+        foreach ($in as $sourceId => $source) {
+            if ($in->hasPermission($permission)) {
+                $out[$sourceId] = $source;
+            }
+        }
+
+        return $out;
+    }
+
     /**
      * Returns the default tasklist for the current user at the specified
      * permissions level.
@@ -321,8 +322,9 @@ class Nag {
         if (isset($tasklists[$default_tasklist])) {
             return $default_tasklist;
         } elseif ($prefs->isLocked('default_tasklist')) {
-            return '';
+            return Auth::getAuth();
         } elseif (count($tasklists)) {
+            reset($tasklists);
             return key($tasklists);
         }
 
@@ -353,14 +355,14 @@ class Nag {
     }
 
     /**
-     * Builds the HTML for a task completion state widget.
+     * Builds the HTML for a checkbox widget.
      *
      * @param string $name      The name of the widget.
-     * @param integer $checked  The default completion state.
+     * @param integer $checked  The default checkbox state.
      *
      * @return string  HTML for a checkbox representing the completion state.
      */
-    function buildCompletionWidget($name, $checked = 0)
+    function buildCheckboxWidget($name, $checked = 0)
     {
         $name = htmlspecialchars($name);
         return "<input type=\"checkbox\" id=\"$name\" name=\"$name\"" .
@@ -371,10 +373,11 @@ class Nag {
      * Formats the given Unix-style date string.
      *
      * @param string $unixdate  The Unix-style date value to format.
+     * @param boolean $hours    Whether to add hours.
      *
      * @return string  The formatted due date string.
      */
-    function formatDate($unixdate = '')
+    function formatDate($unixdate = '', $hours = true)
     {
         global $prefs;
 
@@ -382,8 +385,13 @@ class Nag {
             return '';
         }
 
+        $date = strftime($prefs->getValue('date_format'), $unixdate);
+        if (!$hours) {
+            return $date;
+        }
+
         return sprintf(_("%s at %s"),
-                       strftime($prefs->getValue('date_format'), $unixdate),
+                       $date,
                        strftime($prefs->getValue('twentyFour') ? '%H:%M' : '%I:%M %p', $unixdate));
     }
 
@@ -465,7 +473,7 @@ class Nag {
         if (is_array($allowed)) {
             switch ($permission) {
             case 'max_tasks':
-                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
+                $allowed = max($allowed);
                 break;
             }
         }
@@ -473,6 +481,81 @@ class Nag {
         return $allowed;
     }
 
+    /**
+     * Initial app setup code.
+     */
+    function initialize()
+    {
+        /* Store the request timestamp if it's not already present. */
+        if (!isset($_SERVER['REQUEST_TIME'])) {
+            $_SERVER['REQUEST_TIME'] = time();
+        }
+
+        // Update the preference for what task lists to display. If the user
+        // doesn't have any selected task lists for view then fall back to
+        // some available list.
+        $GLOBALS['display_tasklists'] = @unserialize($GLOBALS['prefs']->getValue('display_tasklists'));
+        if (!$GLOBALS['display_tasklists']) {
+            $GLOBALS['display_tasklists'] = array();
+        }
+        if (($tasklistId = Util::getFormData('display_tasklist')) !== null) {
+            if (is_array($tasklistId)) {
+                $GLOBALS['display_tasklists'] = $tasklistId;
+            } else {
+                if (in_array($tasklistId, $GLOBALS['display_tasklists'])) {
+                    $key = array_search($tasklistId, $GLOBALS['display_tasklists']);
+                    unset($GLOBALS['display_tasklists'][$key]);
+                } else {
+                    $GLOBALS['display_tasklists'][] = $tasklistId;
+                }
+            }
+        }
+
+        // Make sure all task lists exist now, to save on checking later.
+        $_temp = $GLOBALS['display_tasklists'];
+        $GLOBALS['all_tasklists'] = Nag::listTasklists();
+        $GLOBALS['display_tasklists'] = array();
+        foreach ($_temp as $id) {
+            if (isset($GLOBALS['all_tasklists'][$id])) {
+                $GLOBALS['display_tasklists'][] = $id;
+            }
+        }
+
+        if (count($GLOBALS['display_tasklists']) == 0) {
+            $lists = Nag::listTasklists(true);
+            if (!Auth::getAuth()) {
+                /* All tasklists for guests. */
+                $GLOBALS['display_tasklists'] = array_keys($lists);
+            } else {
+                /* Make sure at least the default tasklist is visible. */
+                $default_tasklist = Nag::getDefaultTasklist(PERMS_READ);
+                if ($default_tasklist) {
+                    $GLOBALS['display_tasklists'] = array($default_tasklist);
+                }
+
+                /* If the user's personal tasklist doesn't exist, then create it. */
+                if (!$GLOBALS['nag_shares']->exists(Auth::getAuth())) {
+                    require_once 'Horde/Identity.php';
+                    $identity = &Identity::singleton();
+                    $name = $identity->getValue('fullname');
+                    if (trim($name) == '') {
+                        $name = Auth::removeHook(Auth::getAuth());
+                    }
+                    $share = &$GLOBALS['nag_shares']->newShare(Auth::getAuth());
+                    $share->set('name', sprintf(_("%s's Task List"), $name));
+                    $GLOBALS['nag_shares']->addShare($share);
+
+                    /* Make sure the personal tasklist is displayed by default. */
+                    if (!in_array(Auth::getAuth(), $GLOBALS['display_tasklists'])) {
+                        $GLOBALS['display_tasklists'][] = Auth::getAuth();
+                    }
+                }
+            }
+        }
+
+        $GLOBALS['prefs']->setValue('display_tasklists', serialize($GLOBALS['display_tasklists']));
+    }
+
     /**
      * Build Nag's list of menu items.
      */
@@ -482,7 +565,7 @@ class Nag {
 
         require_once 'Horde/Menu.php';
 
-        $menu = &new Menu();
+        $menu = new Menu();
         $menu->add(Horde::applicationUrl('list.php'), _("_List Tasks"), 'nag.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
         if (Nag::getDefaultTasklist(PERMS_EDIT) &&
             (!empty($conf['hooks']['permsdenied']) ||
@@ -490,11 +573,9 @@ class Nag {
              Nag::hasPermission('max_tasks') > Nag::countTasks())) {
             $menu->add(Horde::applicationUrl(Util::addParameter('task.php', 'actionID', 'add_task')), _("_New Task"), 'add.png', null, null, null, Util::getFormData('task') ? '__noselection' : null);
         }
-        $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
 
-        if (Auth::getAuth()) {
-            $menu->add(Horde::applicationUrl('tasklists.php'), _("_My Tasklists"), 'tasklists.png');
-        }
+        /* Search. */
+        $menu->add(Horde::applicationUrl('search.php'), _("_Search"), 'search.png', $registry->getImageDir('horde'));
 
         /* Import/Export. */
         if ($conf['menu']['import_export']) {
@@ -503,7 +584,7 @@ class Nag {
 
         /* Print. */
         if ($conf['menu']['print'] && isset($print_link)) {
-            $menu->add($print_link, _("_Print"), 'print.png', $registry->getImageDir('horde'), '_blank', 'popup(this.href); return false;');
+            $menu->add($print_link, _("_Print"), 'print.png', $registry->getImageDir('horde'), '_blank', 'popup(this.href); return false;', '__noselection');
         }
 
         if ($returnType == 'object') {
@@ -517,31 +598,33 @@ class Nag {
     {
         global $notification;
 
-        // Get any alarms in the next hour.
-        $_now = time();
-        $_alarmList = Nag::listAlarms($_now);
-        if (is_a($_alarmList, 'PEAR_Error')) {
-            Horde::logMessage($_alarmList, __FILE__, __LINE__, PEAR_LOG_ERR);
-            $notification->push($_alarmList, 'horde.error');
-        } else {
-            $_messages = array();
-            foreach ($_alarmList as $_task) {
-                $differential = $_task['due'] - $_now;
-                $_key = $differential;
-                while (isset($_messages[$_key])) {
-                    $_key++;
+        if (empty($GLOBALS['conf']['alarms']['driver'])) {
+            // Get any alarms in the next hour.
+            $now = time();
+            $alarmList = Nag::listAlarms($now);
+            if (is_a($alarmList, 'PEAR_Error')) {
+                Horde::logMessage($alarmList, __FILE__, __LINE__, PEAR_LOG_ERR);
+                $notification->push($alarmList, 'horde.error');
+            } else {
+                $messages = array();
+                foreach ($alarmList as $task) {
+                    $differential = $task->due - $now;
+                    $key = $differential;
+                    while (isset($messages[$key])) {
+                        $key++;
+                    }
+                    if ($differential >= -60 && $differential < 60) {
+                        $messages[$key] = array(sprintf(_("%s is due now."), $task->name), 'nag.alarm');
+                    } elseif ($differential >= 60) {
+                        $messages[$key] = array(sprintf(_("%s is due in %s"), $task->name,
+                                                        Nag::secondsToString($differential)), 'nag.alarm');
+                    }
                 }
-                if ($differential >= -60 && $differential < 60) {
-                    $_messages[$_key] = array(sprintf(_("%s is due now."), $_task['name']), 'nag.alarm');
-                } elseif ($differential >= 60) {
-                    $_messages[$_key] = array(sprintf(_("%s is due in %s"), $_task['name'],
-                                                      Nag::secondsToString($differential)), 'nag.alarm');
-                }
-            }
 
-            ksort($_messages);
-            foreach ($_messages as $message) {
-                $notification->push($message[0], $message[1]);
+                ksort($messages);
+                foreach ($messages as $message) {
+                    $notification->push($message[0], $message[1]);
+                }
             }
         }
 
@@ -619,6 +702,9 @@ class Nag {
         }
         foreach ($share->listGroups(PERMS_READ) as $group) {
             $group = $groups->getGroupById($group);
+            if (is_a($group, 'PEAR_Error')) {
+                continue;
+            }
             $group_users = $group->listAllUsers();
             if (is_a($group_users, 'PEAR_Error')) {
                 Horde::logMessage($group_users, __FILE__, __LINE__, PEAR_LOG_ERR);
@@ -647,7 +733,7 @@ class Nag {
             return;
         }
 
-        $msg_headers = &new MIME_Headers();
+        $msg_headers = new MIME_Headers();
         $msg_headers->addMessageIdHeader();
         $msg_headers->addAgentHeader();
         $msg_headers->addHeader('Date', date('r'));
@@ -656,8 +742,8 @@ class Nag {
 
         $message = "\n" . sprintf($notification_message, $name, $share->get('name'), $due ? strftime('%x %X', $due) : 'no due date' . "\n\n" . $desc);
 
-        $mime = &new MIME_Message();
-        $body = &new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
+        $mime = new MIME_Message();
+        $body = new MIME_Part('text/plain', String::wrap($message, 76, "\n"), NLS::getCharset());
 
         $mime->addPart($body);
         $msg_headers->addMIMEHeaders($mime);
@@ -674,6 +760,28 @@ class Nag {
         return $mime->send(implode(', ', $addresses), $msg_headers, $mail_driver, $mail_params);
     }
 
+    /**
+     * Returns the real name, if available, of a user.
+     *
+     * @since Nag 2.2
+     */
+    function getUserName($uid)
+    {
+        static $names = array();
+
+        if (!isset($names[$uid])) {
+            require_once 'Horde/Identity.php';
+            $ident = &Identity::singleton('none', $uid);
+            $ident->setDefault($ident->getDefault());
+            $names[$uid] = $ident->getValue('fullname');
+            if (empty($names[$uid])) {
+                $names[$uid] = $uid;
+            }
+        }
+
+        return $names[$uid];
+    }
+
     /**
      * Returns whether a user wants email notifications for a tasklist.
      *
@@ -709,7 +817,7 @@ class Nag {
             return $mode == 'read';
         case 'show':
             if ($mode == 'read') {
-                $display_tasklists = unserialize($prefs->getValue('display_tasklsts'));
+                $display_tasklists = unserialize($prefs->getValue('display_tasklists'));
                 return in_array($tasklist, $display_tasklists);
             }
         }
@@ -728,10 +836,10 @@ class Nag {
      */
     function _sortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) {
+        if ($a->priority == $b->priority) {
             return 0;
         }
-        return ($a['priority'] > $b['priority']) ? 1 : -1;
+        return ($a->priority > $b->priority) ? 1 : -1;
     }
 
     /**
@@ -745,10 +853,10 @@ class Nag {
      */
     function _rsortByPriority($a, $b)
     {
-        if ($a['priority'] == $b['priority']) {
+        if ($a->priority == $b->priority) {
             return 0;
         }
-        return ($a['priority'] > $b['priority']) ? -1 : 1;
+        return ($a->priority > $b->priority) ? -1 : 1;
     }
 
     /**
@@ -762,7 +870,7 @@ class Nag {
      */
     function _sortByName($a, $b)
     {
-        return strcasecmp($a['name'], $b['name']);
+        return strcasecmp($a->name, $b->name);
     }
 
     /**
@@ -776,7 +884,7 @@ class Nag {
      */
     function _rsortByName($a, $b)
     {
-        return strcasecmp($b['name'], $a['name']);
+        return strcasecmp($b->name, $a->name);
     }
 
     /**
@@ -790,8 +898,8 @@ class Nag {
      */
     function _sortByCategory($a, $b)
     {
-        return strcasecmp($a['category'] ? $a['category'] : _("Unfiled"),
-                          $b['category'] ? $b['category'] : _("Unfiled"));
+        return strcasecmp($a->category ? $a->category : _("Unfiled"),
+                          $b->category ? $b->category : _("Unfiled"));
     }
 
     /**
@@ -805,8 +913,8 @@ class Nag {
      */
     function _rsortByCategory($a, $b)
     {
-        return strcasecmp($b['category'] ? $b['category'] : _("Unfiled"),
-                          $a['category'] ? $a['category'] : _("Unfiled"));
+        return strcasecmp($b->category ? $b->category : _("Unfiled"),
+                          $a->category ? $a->category : _("Unfiled"));
     }
 
     /**
@@ -820,19 +928,19 @@ class Nag {
      */
     function _sortByDue($a, $b)
     {
-        if ($a['due'] == $b['due']) {
+        if ($a->due == $b->due) {
             return 0;
         }
 
         // Treat empty due dates as farthest into the future.
-        if ($a['due'] == 0) {
+        if ($a->due == 0) {
             return 1;
         }
-        if ($b['due'] == 0) {
+        if ($b->due == 0) {
             return -1;
         }
 
-        return ($a['due'] > $b['due']) ? 1 : -1;
+        return ($a->due > $b->due) ? 1 : -1;
     }
 
     /**
@@ -846,19 +954,19 @@ class Nag {
      */
     function _rsortByDue($a, $b)
     {
-        if ($a['due'] == $b['due']) {
+        if ($a->due == $b->due) {
             return 0;
         }
 
         // Treat empty due dates as farthest into the future.
-        if ($a['due'] == 0) {
+        if ($a->due == 0) {
             return -1;
         }
-        if ($b['due'] == 0) {
+        if ($b->due == 0) {
             return 1;
         }
 
-        return ($a['due'] < $b['due']) ? 1 : -1;
+        return ($a->due < $b->due) ? 1 : -1;
     }
 
     /**
@@ -872,10 +980,10 @@ class Nag {
      */
     function _sortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) {
+        if ($a->completed == $b->completed) {
             return 0;
         }
-        return ($a['completed'] > $b['completed']) ? -1 : 1;
+        return ($a->completed > $b->completed) ? -1 : 1;
     }
 
     /**
@@ -889,10 +997,10 @@ class Nag {
      */
     function _rsortByCompletion($a, $b)
     {
-        if ($a['completed'] == $b['completed']) {
+        if ($a->completed == $b->completed) {
             return 0;
         }
-        return ($a['completed'] < $b['completed']) ? -1 : 1;
+        return ($a->completed < $b->completed) ? -1 : 1;
     }
 
     /**
@@ -906,11 +1014,11 @@ class Nag {
      */
     function _sortByOwner($a, $b)
     {
-        $ashare = $GLOBALS['nag_shares']->getShare($a['tasklist_id']);
-        $bshare = $GLOBALS['nag_shares']->getShare($b['tasklist_id']);
+        $ashare = $GLOBALS['nag_shares']->getShare($a->tasklist);
+        $bshare = $GLOBALS['nag_shares']->getShare($b->tasklist);
 
-        $aowner = $a['tasklist_id'];
-        $bowner = $b['tasklist_id'];
+        $aowner = $a->tasklist;
+        $bowner = $b->tasklist;
 
         if (!is_a($ashare, 'PEAR_Error') && $aowner != $ashare->get('owner')) {
             $aowner = $ashare->get('name');
@@ -933,11 +1041,11 @@ class Nag {
      */
     function _rsortByOwner($a, $b)
     {
-        $ashare = $GLOBALS['nag_shares']->getShare($a['tasklist_id']);
-        $bshare = $GLOBALS['nag_shares']->getShare($b['tasklist_id']);
+        $ashare = $GLOBALS['nag_shares']->getShare($a->tasklist);
+        $bshare = $GLOBALS['nag_shares']->getShare($b->tasklist);
 
-        $aowner = $a['tasklist_id'];
-        $bowner = $b['tasklist_id'];
+        $aowner = $a->tasklist;
+        $bowner = $b->tasklist;
 
         if (!is_a($ashare, 'PEAR_Error') && $aowner != $ashare->get('owner')) {
             $aowner = $ashare->get('name');
