@@ -1,11 +1,13 @@
 <?php
 /**
- * $Horde: nag/themes/categoryCSS.php,v 1.1.2.3 2007/01/02 13:55:13 jan Exp $
+ * $Horde: nag/themes/categoryCSS.php,v 1.1.2.6 2008/05/20 21:59:32 jan Exp $
  *
- * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (GPL).  If you
+ * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 @define('AUTH_HANDLER', true);
@@ -16,7 +18,7 @@ require_once 'Horde/Prefs/CategoryManager.php';
 
 header('Content-Type: text/css');
 
-$cManager = &new Prefs_CategoryManager();
+$cManager = new Prefs_CategoryManager();
 
 $colors = $cManager->colors();
 $fgColors = $cManager->fgColors();
@@ -27,7 +29,8 @@ foreach ($colors as $category => $color) {
 
     $class = '.category' . md5($category);
 
-    echo "$class, .linedRow td$class, .overdue td$class, .closed td$class { ",
-        'color: ' . (isset($fgColors[$category]) ? $fgColors[$category] : $fgColors['_default_']) . '; ',
-        'background: ' . $color . "; }\n";
+    echo "$class, .linedRow td$class, .overdue td$class, .closed td$class { "
+        . 'color: ' . (isset($fgColors[$category]) ? $fgColors[$category] : $fgColors['_default_']) . '; '
+        . 'background: ' . $color . '; '
+        . "padding: 0 4px; }\n";
 }
